<?php

/**
 * @file
 * Facade for Smartling APIs.
 */

namespace Drupal\smartling\ApiWrapper;

use Drupal\smartling\ApiWrapperInterface;
use Drupal\smartling\Log\SmartlingLog;
use Drupal\smartling\Settings\SmartlingSettingsHandler;
use Exception;
use HtmlSaveComplete;
use Smartling\BaseApiAbstract;
use Smartling\Context\ContextApi;
use Smartling\Context\Params\UploadContextParameters;
use Smartling\Context\Params\UploadResourceParameters;
use Smartling\Exceptions\SmartlingApiException;
use Smartling\File\FileApi;
use Smartling\File\Params\DownloadFileParameters;
use Smartling\File\Params\UploadFileParameters;
use Smartling\Project\ProjectApi;

/**
 * Class SmartlingApiWrapper.
 */
class SmartlingApiWrapper implements ApiWrapperInterface {

  /**
   * @var SmartlingSettingsHandler
   */
  protected $settingsHandler;

  /**
   * @var SmartlingLog
   */
  protected $logger;

  /**
   * @var FileApi
   */
  protected $api;

  /**
   * @var ProjectApi
   */
  protected $project_api;

  /**
   * @var ContextApi
   */
  protected $context_api;

  // TODO: uncomment when JobsAPI will be integrated.
  // /**
  //  * @var JobsApi
  //  */
  // protected $jobs_api;

  /**
   * This function converts Drupal locale to Smartling locale.
   *
   * @param string $locale
   *   Locale string in some format: 'en' or 'en-US'.
   *
   * @return string|null
   *   Return locale or NULL.
   */
  protected function convertLocaleDrupalToSmartling($locale) {
    return smartling_convert_locale_drupal_to_smartling($locale);
  }

  /**
   * This function converts Smartling locale to Drupal locale.
   *
   * @param string $locale
   *   Locale string in some format: 'en' or 'en-US'.
   *
   * @return string|null
   *   Return locale or NULL.
   */
  protected function convertLocaleSmartlingToDrupal($locale) {
    return smartling_convert_locale_smartling_to_drupal($locale);
  }
  /**
   * Initialize.
   *
   * @param SmartlingSettingsHandler $settings_handler
   * @param SmartlingLog $logger
   */
  public function __construct(SmartlingSettingsHandler $settings_handler, SmartlingLog $logger) {
    $this->settingsHandler = $settings_handler;
    $this->logger = $logger;
    $api_factory = drupal_container()->get('smartling.smartling_api_factory');

    $this->setApi($api_factory::create($this->settingsHandler, 'project'), 'project');
    $this->setApi($api_factory::create($this->settingsHandler, 'context'), 'context');

    // TODO: uncomment when JobsAPI will be integrated.
    // $this->setApi($api_factory::create($this->settingsHandler, 'jobs'), 'jobs');

    $this->setApi($api_factory::create($this->settingsHandler));
  }

  /**
   * {@inheritdoc}
   *
   * We don't use type hint as BaseApiAbstract because we have
   * SmartlingContextAPI class that isn't extended from BaseApiAbstract.
   */
  public function setApi($api, $api_type = 'file') {
    switch ($api_type) {
      case 'file':
        $this->api = $api;

        break;

      case 'project':
        $this->project_api = $api;

        break;

      // TODO: uncomment when JobsAPI will be integrated.
      // case 'jobs':
      // $this->jobs_api = $api;
      //
      // break;

      case 'context':
        $this->context_api = $api;

        break;
    }
  }

  // TODO: 3.x - never used?
  public function getLocaleList() {
    $response = $this->api->getLocaleList();
    $response = json_decode($response);
    $locales = isset($response->response->data->locales) ? $response->response->data->locales : array();
    $result = array();
    foreach ($locales as $locale) {
      $result[$locale->locale] = "{$locale->name} ({$locale->translated})";
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function downloadFile($smartling_entity) {
    $smartling_entity_type = $smartling_entity->entity_type;
    $d_locale = $smartling_entity->target_language;
    $file_name_unic = $smartling_entity->file_name;

    $retrieval_type = $this->settingsHandler->variableGet('smartling_retrieval_type', 'published');

    $download_param = new DownloadFileParameters();
    $download_param->setRetrievalType($retrieval_type);

    $this->logger->info("Smartling queue start download '@file_name' file and update fields for @entity_type id - @rid, locale - @locale.",
      array(
        '@file_name' => $file_name_unic,
        '@entity_type' => $smartling_entity_type,
        '@rid' => $smartling_entity->rid,
        '@locale' => $smartling_entity->target_language
      ));

    $s_locale = $this->convertLocaleDrupalToSmartling($d_locale);

    // Try to download file.
    try {
      $download_result = (string) $this->api->downloadFile($file_name_unic, $s_locale, $download_param);
    }
    catch (SmartlingApiException $e) {
      $this->logger->error('smartling_queue_download_translated_item_process tried to download file:<br/>
      Project Id: @project_id <br/>
      Action: download <br/>
      URI: @file_uri <br/>
      Drupal Locale: @d_locale <br/>
      Smartling Locale: @s_locale <br/>
      Error: @message',
        array(
          '@project_id' => $this->settingsHandler->getProjectId(),
          '@file_uri' => $file_name_unic,
          '@d_locale' => $d_locale,
          '@s_locale' => $s_locale,
          '@message' => $e->getMessage(),
        ), TRUE);

      $download_result = FALSE;
    }

    return $download_result;
  }


  /**
   * {@inheritdoc}
   */
  public function getStatus($smartling_entity) {
    $error_result = NULL;

    if ($smartling_entity === FALSE) {
      $this->logger->error('Smartling checks status for id - @rid is FAIL! Smartling entity not exist.', array('@rid' => $smartling_entity->rid), TRUE);
      return $error_result;
    }

    $file_name_unic = $smartling_entity->file_name;

    $s_locale = $this->convertLocaleDrupalToSmartling($smartling_entity->target_language);

    // Try to retrieve file status.
    try {
      $status_result = $this->api->getStatus($file_name_unic, $s_locale);
    }
    catch (SmartlingApiException $e) {
      $this->logger->error('Smartling checks status for @entity_type id - @rid: <br/>
      Project Id: @project_id <br/>
      Action: status <br/>
      URI: @file_uri <br/>
      Drupal Locale: @d_locale <br/>
      Smartling Locale: @s_locale <br/>
      Error: @message', array(
        '@entity_type' => $smartling_entity->entity_type,
        '@rid' => $smartling_entity->rid,
        '@project_id' => $this->settingsHandler->getProjectId(),
        '@file_uri' => $file_name_unic,
        '@d_locale' => $smartling_entity->target_language,
        '@s_locale' => $s_locale,
        '@message' => $e->getMessage(),
      ), TRUE);

      return $error_result;
    }

    $this->logger->info('Smartling checks status for @entity_type id - @rid (@d_locale). approvedString = @as, completedString = @cs',
      array(
        '@entity_type' => $smartling_entity->entity_type,
        '@rid' => $smartling_entity->rid,
        '@d_locale' => $smartling_entity->target_language,
        '@as' => $status_result['authorizedStringCount'],
        '@cs' => $status_result['completedStringCount'],
      ));

    // If true, file translated.
    $authorized = $status_result['authorizedStringCount'];
    $completed = $status_result['completedStringCount'];
    $progress = ($authorized + $completed > 0) ? (int) (($completed / ($authorized + $completed)) * 100) : 0;
    $smartling_entity->download = 0;
    $smartling_entity->progress = $progress;
    $smartling_entity->status = SMARTLING_STATUS_IN_TRANSLATE;

    return array(
      'entity_data' => $smartling_entity,
      'response_data' => $status_result,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function testConnection(array $locales) {
    $result = array();

    try {
      $project_details = $this->project_api->getProjectDetails();
      $targetLocales = [];

      foreach ($project_details['targetLocales'] as $targetLocale) {
        if (!empty($targetLocale['enabled'])) {
          $targetLocales[] = $targetLocale['localeId'];
        }
      }

      foreach ($locales as $key => $locale) {
        if ($locale !== 0 && $locale == $key) {
          $s_locale = $this->convertLocaleDrupalToSmartling($locale);
          $result[$s_locale] = FALSE;

          if (in_array($s_locale, $targetLocales)) {
            $result[$s_locale] = TRUE;
          }
        }
      }
    }
    catch (SmartlingApiException $e) {
      $this->logger->warning('Connection test for project: @project_id FAILED.',
        array(
          '@project_id' => $this->settingsHandler->getProjectId(),
        ));
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  // TODO : Replace $file_type with enum class
  public function uploadFile($file_path, $file_name_unic, $file_type, array $locales) {
    $locales_to_approve = array();
    foreach ($locales as $locale) {
      $locales_to_approve[] = $this->convertLocaleDrupalToSmartling($locale);
    }

    $version = system_get_info('module', 'smartling');
    $version = $version['version'];

    $upload_params = new UploadFileParameters(
      BaseApiAbstract::CLIENT_LIB_ID_SDK,
      $version
    );

    if ($this->settingsHandler->getAutoAuthorizeContent()) {
      $upload_params->setLocalesToApprove($locales_to_approve);
    }
    if ($this->settingsHandler->getCallbackUrlUse()) {
      $upload_params->setCallbackUrl($this->settingsHandler->getCallbackUrl());
    }
    if ($this->settingsHandler->getOverwriteApprovedLocales()) {
      $upload_params->set('overwriteAuthorizedLocales', 1);
    }

    try {
      $this->api->uploadFile($file_path, $file_name_unic, $file_type, $upload_params);

      $this->logger->info('Smartling uploaded @file_name for locales: @locales',
        array(
          '@file_name' => $file_name_unic,
          '@locales' => implode('; ', $locales),
          'entity_link' => l(t('View file'), $file_path)
        ));

      return SMARTLING_STATUS_EVENT_UPLOAD_TO_SERVICE;
    }
    catch (SmartlingApiException $e) {
      $this->logger->error('Smartling failed to upload xml file: <br/>
          Project Id: @project_id <br/>
          Action: upload <br/>
          URI: @file_uri <br/>
          Drupal Locale: @d_locale <br/>
          Smartling Locale: @s_locale <br/>
          Error: @message <br/>
          Upload params: @upload_params',
        array(
          '@project_id' => $this->settingsHandler->getProjectId(),
          '@file_uri' => $file_path,
          '@d_locale' => implode('; ', $locales),
          '@s_locale' => implode('; ', $locales_to_approve),
          '@message' => $e->getMessage(),
          '@upload_params' => print_r($upload_params->exportToArray(), TRUE),
        ), TRUE);
    }

    return SMARTLING_STATUS_EVENT_FAILED_UPLOAD;
  }

  /**
   * Uploads context file to Smartling and writes some logs.
   *
   * @param array $data
   * @return int
   */
  public function uploadContext($data) {
    $data['action'] = 'upload';
    $result = FALSE;

    try {
      $context_file_name = str_replace(['/', '=', '.'], '_', implode('_', parse_url($data['url']))) . '.html';
      $path = $this->settingsHandler->getDir() . '/smartling_context/' . $context_file_name;
      $dirname = dirname($path);

      if (file_prepare_directory($dirname, FILE_CREATE_DIRECTORY) && ($file = file_save_data($data['html'], $path, FILE_EXISTS_REPLACE))) {
        $path = drupal_realpath($file->uri);
        $params = new UploadContextParameters();
        $params->setContextFileUri($path);
        $params->setName($data['url']);

        $this->context_api->uploadAndMatchContextSync($params);
        $this->uploadContextMissingResources($dirname, [
          'basic_auth' => [
            'enabled' => variable_get('smartling_translation_context_basic_auth_enabled', FALSE),
            'login' => variable_get('smartling_translation_context_basic_auth_login', ''),
            'password' => variable_get('smartling_translation_context_basic_auth_password', ''),
          ]
        ]);

        $result = TRUE;
      }
      else {
        $this->logger->error("Can't save context file: @path", [
          '@path' => $path,
        ]);
      }
    }
    catch (SmartlingApiException $e) {
      $result = FALSE;
      $this->logger->error('Smartling failed to upload context for @url with message: @message', array(
        '@url' => $data['url'],
        '@message' => $e->getMessage()
      ), TRUE);
    }

    return $result;
  }

  /**
   * @param $smartling_context_directory
   * @param array $settings
   */
  protected function uploadContextMissingResources($smartling_context_directory, array $settings) {
    // Cache for resources which we can't upload. Do not try to re-upload them
    // for 1 hour. After 1 hour cache will be reset and we will try again.
    $cache_name = 'smartling_context_resources_cache';
    $time_to_live = 60 * 60;
    $cache = cache_get($cache_name);
    $cached_data = empty($cache) ? [] : $cache->data;
    $update_cache = FALSE;
    $smartling_context_resources_directory = $smartling_context_directory . '/resources';
    $asset_inliner = new HtmlSaveComplete();

    // Do nothing if directory for resources isn't accessible.
    if (!file_prepare_directory($smartling_context_resources_directory, FILE_CREATE_DIRECTORY)) {
      $this->logger->error("Context resources directory @dir doesn't exist or is not writable. Missing resources were not uploaded. Context might look incomplete.", [
        '@dir' => $smartling_context_directory,
      ]);

      return;
    }

    try {
      $time_out = drupal_is_cli() ? 300 : 30;
      $start_time = time();

      do {
        $delta = time() - $start_time;

        if ($delta > $time_out) {
          throw new SmartlingApiException(vsprintf('Not all context resources are uploaded after %s seconds.', [$delta]));
        }

        $all_missing_resources = $this->context_api->getAllMissingResources();

        // Method getAllMissingResources can return not all missing resources
        // in case it took to much time. Log this information.
        if (!$all_missing_resources['all']) {
          $this->logger->warning('Not all missing context resources are received. Context might look incomplete.');
        }

        $fresh_resources = [];

        foreach ($all_missing_resources['items'] as $item) {
          if (!in_array($item['resourceId'], $cached_data)) {
            $fresh_resources[] = $item;
          }
        }

        // Walk through missing resources and try to upload them.
        foreach ($fresh_resources as $item) {
          // If current resource isn't in the cache and it's accessible then
          // it means we can try to upload it.
          if (!in_array($item['resourceId'], $cached_data) && $asset_inliner->remote_file_exists($item['url'], $settings)) {
            $smartling_context_resource_file = $smartling_context_resources_directory . '/' . $item['resourceId'];
            $smartling_context_resource_file_content = $asset_inliner->getUrlContents($settings, $item['url']);

            // Ensure that resources directory is accessible, resource
            // downloaded properly and only then upload it. ContextAPI will not
            // be able to fopen() resource which is behind basic auth. So
            // download it first (with a help of curl), save it to smartling's
            // directory and then upload.
            if ($file = file_save_data($smartling_context_resource_file_content, $smartling_context_resource_file, FILE_EXISTS_REPLACE)) {
              $path = drupal_realpath($file->uri);
              $params = new UploadResourceParameters();
              $params->setFile($path);
              $is_resource_uploaded = $this->context_api->uploadResource($item['resourceId'], $params);

              // Resource isn't uploaded for some reason. Log this info and set
              // resource id into the cache. We will not try to upload this
              // resource for the next hour.
              if (!$is_resource_uploaded) {
                $update_cache = TRUE;
                $cached_data[] = $item['resourceId'];

                $this->logger->warning("Can't upload context resource file with id = @id and url = @url. Context might look incomplete.", [
                  '@id' => $item['resourceId'],
                  '@url' => $item['url'],
                ]);
              }
            }
            // We can't save context resource file. Log this info.
            else {
              $this->logger->error("Can't save context resource file: @path", [
                '@path' => $smartling_context_resource_file,
              ]);
            }
          }
          else {
            // Current resource isn't accessible (or already in the cache).
            // If first case then add inaccessible resource into the cache.
            if (!in_array($item['resourceId'], $cached_data)) {
              $update_cache = TRUE;
              $cached_data[] = $item['resourceId'];

              $this->logger->warning("File @file can not be downloaded.", [
                '@file' => $item['url'],
              ]);
            }
          }
        }

        // Set failed resources into the cache for the next hour.
        if ($update_cache) {
          cache_set($cache_name, $cached_data, 'cache', time() + $time_to_live);
        }
      } while (!empty($fresh_resources));
    } catch (Exception $e) {
      watchdog_exception('smartling', $e);
    }
  }

  function deleteFile($file_name) {
    try {
      $this->api->deleteFile($file_name);
    }
    catch (SmartlingApiException $e) {
      $this->logger->error('Smartling failed to delete file: @file_name', array('@file_name' => $file_name), TRUE);

      return -1;
    }

    return 1;
  }

  /**
   * {@inheritdoc}
   */
  public function getStatusAllLocales($file_name) {
    $error_result = NULL;

    if (empty($file_name)) {
      //@todo: improve log message.
      $this->logger->error('Smartling checks status for file - @file_name failed! Because it was empty.', array('@file_name' => $file_name), TRUE);
      return $error_result;
    }

    // Try to retrieve file status.
    try {
      $status_result = $this->api->getStatusAllLocales($file_name);
    }
    catch (SmartlingApiException $e) {
      $this->logger->error('Smartling checks status for file name: @file_uri <br/>
      Project Id: @project_id <br/>
      Action: status <br/>
      Error: response code -> @code and message -> @message', array(
        '@project_id' => $this->settingsHandler->getProjectId(),
        '@file_uri' => $file_name,
        '@message' => $e->getMessage(),
      ), TRUE);

      return $error_result;
    }

    $this->logger->info('Smartling checks status for file: @file_name. totalString = @as',
      array(
        '@file_name' => $file_name,
        '@as' => $status_result['totalStringCount'],
      ));

    $result = array();
    foreach ($status_result['items'] as $item) {
      $locales = $this->convertLocaleSmartlingToDrupal($item['localeId']);

      foreach ($locales as $locale) {
        $result[$locale] = (array) $item;
      }
    }

    return $result;
  }

  public function getLastModified($file_name) {
    //return $this->api->getLastModified($fileUri, $locale, $params);

    $error_result = NULL;

    if (empty($file_name)) {
      //@todo: improve log message.
      $this->logger->error('Smartling check of last updated date for file - @file_name failed.', array('@file_name' => $file_name), TRUE);
      return $error_result;
    }

    // Try to retrieve file status.
    try {
      $status_result = $this->api->getLastModified($file_name);
    }
    catch (SmartlingApiException $e) {
      $this->logger->error('Smartling check last updated dates for file name: @file_uri <br/>
      Project Id: @project_id <br/>
      Action: status <br/>
      Error: @message', array(
        '@project_id' => $this->settingsHandler->getProjectId(),
        '@file_uri' => $file_name,
        '@message' => $e->getMessage(),
      ), TRUE);
      return $error_result;
    }

    $result = array();
    foreach ($status_result['items'] as $item) {
      $locales = $this->convertLocaleSmartlingToDrupal($item['localeId']);

      foreach ($locales as $locale) {
        $result[$locale] = strtotime($item['lastModified']);
      }
    }

    return $result;
  }
}
